function InitCommanders()

  local id, faction
  local i = 1

  while data.commanders[i] ~= nil do

    id = data.commanders[i].id
    faction = game:GetFactionById(data.commanders[i].faction)

    -- Init commanders table
    if faction ~= nil then
      if faction.luaData.commanders == nil then
        faction.luaData.commanders = {}
      end

      faction.luaData.commanders[id] = {}
      faction.luaData.commanders[id].turns = 0
      faction.luaData.commanders[id].unlocked = 0
      faction.luaData.commanders[id].type = data.commanders[i].type
    end

    i = i + 1

  end

end


function DecreaseFactionCommanderRequirement(factionId, req, class, amount)
  local faction = game:GetFactionById(factionId)
  if faction.luaData.commanderRequirements == nil then
    faction.luaData.commanderRequirements = {}
  end
  if faction.luaData.commanderRequirements[req] == nil then
    faction.luaData.commanderRequirements[req] = {}
  end
  faction.luaData.commanderRequirements[req][class] = amount
end

function GetFactionCommanderRequirementReduction(factionId, req, class)
  local faction = game:GetFactionById(factionId)
  if faction.luaData.commanderRequirements == nil or faction.luaData.commanderRequirements[req] == nil or faction.luaData.commanderRequirements[req][class] == nil then
    return 0
  else
    LogUi("return " .. faction.luaData.commanderRequirements[req][class])
    return faction.luaData.commanderRequirements[req][class]
  end
end

function UnlockCommanders(alliance)
  for faction in alliance.factions do
    if faction.luaData.commanders ~= nil then
      for i, commander in pairs(faction.luaData.commanders) do
        if commander.unlocked == 0 then
          if CommanderConditionsCheck(faction, i)[0] == true then
            commander.unlocked = game.turn
          end
        elseif commander.turns > 0 then
          commander.turns = commander.turns - 1
        end
      end
    end
  end
end

function CommanderConditionsCheck(faction, commander)
  local conditions = {}
  local conditionCounter = 1

  local i = commander + 1

  -- Ignore "removed" commanders
  if data.commanders[i].conditions.removed then
    conditions[0] = false
    return conditions
  end

  -- All conditions true
  conditions[0] = true

  -- Condition: Kills
  if data.commanders[i].conditions.kills ~= nil then
    local reduction = GetFactionCommanderRequirementReduction(faction.id, "kills", data.commanders[i].conditions.kills[2])
    if faction.luaData.statsKills[data.commanders[i].conditions.kills[2]] / 10 >= data.commanders[i].conditions.kills[1] - reduction then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: Casualties
  if data.commanders[i].conditions.casualties ~= nil then
    local reduction = GetFactionCommanderRequirementReduction(faction.id, "casualties", data.commanders[i].conditions.kills[2])
    if faction.luaData.statsCasualties[data.commanders[i].conditions.casualties[2]] / 10 >= data.commanders[i].conditions.casualties[1] - reduction then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: Research
  if data.commanders[i].conditions.researched ~= nil then
    if faction.luaData.researchComplete[data.commanders[i].conditions.researched] > 0 then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: War
  if data.commanders[i].conditions.war ~= nil then
    local alliance = game:GetFactionById(data.commanders[i].conditions.war).alliance
    if faction.alliance.id ~= alliance.id and alliance.id ~= 0 then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: Ally
  if data.commanders[i].conditions.ally ~= nil then
    local alliance = game:GetFactionById(data.commanders[i].conditions.ally).alliance
    if faction.alliance.id == alliance.id and alliance.id ~= 0 then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: Year
  if data.commanders[i].conditions.year ~= nil then
    if game.date.year >= data.commanders[i].conditions.year then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  -- Condition: Unlocked
  if data.commanders[i].conditions.unlocked ~= nil then
    local unlocked = GetCommanderByName(data.commanders[i].conditions.unlocked)
    if faction.luaData.commanders[unlocked] and faction.luaData.commanders[unlocked].unit ~= nil then
      conditions[conditionCounter] = true
    else
      conditions[conditionCounter] = false
      conditions[0] = false
    end
    conditionCounter = conditionCounter + 1
  end

  return conditions
end


-- Add commander to production queue
function AddCommander(id, faction)

  local i = 1

  while data.commanders[i] ~= nil do

    if data.commanders[i].id == id then

      faction = game:GetFactionById(data.commanders[i].faction)

      faction.luaData.commanders[id].unlocked = game.turn

      return true

    end

    i = i + 1

  end
end


function GetCommanderByName(name)

  local i = 1

  while data.commanders[i] ~= nil do
    if data.commanders[i].name == name then
      return data.commanders[i].id
    end
    i = i + 1
  end

  return false
end


--
function CommanderCompatible(commander, unit)
  if unit.faction.id == data.commanders[commander+1].faction and unit.commander == nil then
    if data.commanders[commander+1].type == "land" and unit.prototype.lua.class == "ground" then
      return true
    end
    if data.commanders[commander+1].type == "air" and unit.prototype.lua.class == "air" then
      return true
    end
    if data.commanders[commander+1].type == "naval" and unit.prototype.lua.class == "naval" and GetStat(unit, "navalattack") ~= nil then
      return true
    end
  end
  return false
end


-- Attach commander to unit
function AttachCommanderToUnit(commander, unit, y, source)

  if source == nil and game.turn == -1 then
    source = "init"
  end

  -- Ability to attach commander to unit at (x,y) coords
  if y ~= nil then
    local hex = GetHex(unit, y)
    -- if there is unit and its part of the commander's faction
    if hex.unit ~= nil and hex.unit.faction.luaData.commanders[commander] ~= nil then
      unit = hex.unit
      unit.faction.luaData.commanders[commander].unlocked = 1
--~       print("unlocked")
    else
      print("failed to attach commander " .. commander)
      return false
    end
  end

  if unit ~= nil and unit.commander == nil and unit.faction.luaData.commanders[commander] ~= nil and unit.faction.luaData.commanders[commander].unlocked ~= 0 and unit.faction.luaData.commanders[commander].unit == nil then
    if source == "ai" or source == "init" then
      unit:AttachCommander(commander)
    else
      GetPlayer():AttachCommander(unit, commander)
    end
    unit.faction.luaData.commanders[commander].unit = unit.id
    MoveCommander(unit)
    EasterEgg_RedBaron(unit)
    return true
  else
    print("failed to attach commander " .. commander)
    return false
  end
end


function MarkCommanderRange(unit, commander)
  if commander == nil then
    commander = unit.commander.id
  end

  local validHexes = {}
  local range = data.commanders[commander+1].range
  for x = -range, range do
    for y = -range, range do
      if unit.hex.x + x >= 0 and unit.hex.x + x <= xSize and unit.hex.y + y >=0 and unit.hex.y + y <= ySize then
        local hex = GetHex(unit.hex.x + x, unit.hex.y + y)
        if unit.hex:Distance(hex) <= range and hex.id ~= unit.hex.id then
          table.insert(validHexes, hex)
          --View:AddEffect(hex, "commander_range_" .. unit.alliance.id)
        end
      end
    end
  end
  --View:AddEffect(hex, "commander_range_" .. unit.alliance.id)
  GetScene():AddActionImages(validHexes, "commander_range_" .. unit.alliance.id)
end


function GetNearbyCommander(unit)
  local range = 4

  -- Transported units dont show commander
  if unit.hex == nil then
    return nil
  end

  -- If unit has its own commander, look no further
  if unit.commander ~= nil then
    return unit.commander
  end

  for x = -range, range do
    for y = -range, range do
      if unit.hex.x + x >= 0 and unit.hex.x + x <= xSize and unit.hex.y + y >=0 and unit.hex.y + y <= ySize then
        local hex = GetHex(unit.hex.x + x, unit.hex.y + y)
        if hex ~= nil and hex.unit ~= nil then
          if hex.unit.faction.id == unit.faction.id and hex.unit.commander ~= nil and CommanderCompatible(hex.unit.commander.id, unit) then
            if unit.hex:Distance(hex) <= data.commanders[hex.unit.commander.id+1].range then
              --unit.commanderInfluence = hex.unit.commander
              return hex.unit.commander
            end
          end
        end
      end
    end
  end

  return nil
end

function GetCommanderBonus(commander, stat)
  if commander ~= nil then
    local bonus = 0
    if data.commanders[commander.id+1].stats ~= nil then
      bonus = data.commanders[commander.id+1].stats[stat]
    end
    if bonus ~= nil then
      return bonus
    end
  end
  return 0
end


function DetachCommanderFromUnit(unit, commander, forceDetach, source)
  local faction = game:GetFactionById(data.commanders[commander.id+1].faction)
  -- Only if parent unit is in full supply
  if unit.supply == 2 or forceDetach == true then
    --unit:DetachCommander()
    if source == "ai" then
      unit:DetachCommander()
    else
      GetPlayer():DetachCommander(unit)
    end

    faction.luaData.commanders[commander.id].unit = nil
    faction.luaData.commanders[commander.id].turns = 1
  end
end


function WoundCommander(commander, recoverTime)
  local faction = game:GetFactionById(data.commanders[commander.id+1].faction)
  if recoverTime == nil then
    recoverTime = 3
  end
  faction.luaData.commanders[commander.id].unit = nil
  faction.luaData.commanders[commander.id].turns = math.random(recoverTime, recoverTime + 2)
end
