function OnUnitNotMoved(unit)
  print("unit was blocked" .. unit.id)
  if unit.hex ~= nil and playerAlliance:IsVisible(unit.hex) then
    MoveSound(unit, 0)

    if unit.alliance.id ~= playerAlliance.id and unit.hex ~= nil then
      followTurn(unit.hex)
    end

    if unit.alive and unit.faction.alliance.id == playerAlliance.id and game.currentAlliance.id == playerAlliance.id and not ui.disableControls then
      ui.SelectHex(unit.hex)
    end
  end
end

function undoSave(name, undo, table)
  undo:Set(name, table[name])
end

function OnUnitMoved(unit, from, moveType, path, undo)
  local to = path[#path]

  if to ~= nil and unit.alliance.id ~= playerAlliance.id or (moveType == 3 and from.faction ~= nil and from.faction.alliance.id ~= playerAlliance.id) then
    if unit.visible then
      if moveType == 2 then
        showMovement(from,to,MoveLogType.Swap,path)
      elseif moveType == 3 and unit.ap == 0 and unit.range == 1 then
        showAttack(from,to,MoveLogType.Advance,path)
      elseif moveType == 1 then
        showMovement(from,to,MoveLogType.Rail,path)
      else
        showMovement(from,to,nil,path)
      end
    end
  end

  undo:SetBool("railMove", moveType == 1)
  -- Consume Railroad move
  if moveType == 1 then
    undoSave("railmovesCurrent", undo, unit.faction.luaData)
    UseRailmove(unit.faction)
    if unit.faction.alliance.id == playerAlliance.id then
      ui.resourcesPanel:Refresh()
    end

    undoSave("repaired", undo, unit.luaData)
    undoSave("upgraded", undo, unit.luaData)
    
    -- Can't upgrade/repair after railmove
    unit.luaData.repaired = game.turn
    unit.luaData.upgraded = game.turn
  end

  undo:SetInt("ap", unit.ap)
  -- Railroad moves OR Artillery & Air lose ability to attack
  if not CanUnitAttackAfterMove(unit) or (moveType == 1 and unit.prototype.lua.chassis ~= "rail") then
    unit.ap = 0
  end

  -- When unit has moved
  if (from ~= nil) then
    -- Clear advance hex
    if gameplay.advanceUnit ~= nil and gameplay.advanceUnit.faction.alliance.id == unit.faction.alliance.id then
      undo:SetInt("gameplay.advanceUnit", gameplay.advanceUnit.id)
      undo:SetInt("gameplay.advanceHex", gameplay.advanceHex.id)
      gameplay.advanceUnit = nil
      gameplay.advanceHex = nil
    end

    if unit.visible and (to ~= nil and playerAlliance:IsVisible(to)) or playerAlliance:IsVisible(from) then
      MoveSound(unit, moveType)
    end
  end

  --MoveCommander(unit, from)

  --MoveConvoy(unit)

  -- Should be move to UI OnUnitMoved!
  if unit.alive and unit.faction.alliance.id == playerAlliance.id and game.currentAlliance.id == playerAlliance.id and not ui.disableControls then
    if ui.swappedHex == nil then
      ui.SelectHex(unit.hex)
    else
      ui.SelectHex(ui.swappedHex)
    end
    RefreshUnitStats(unit, "stats_panel", 1)
  end

  if unit.alive then
    undo:SetInt("efficiency", unit.efficiency)
    DamageEfficiency(unit, GetMoveEfficiencyLoss(unit, to, moveType == 1))

    undoSave("lastAction", undo, unit.luaData)
    undoSave("lastActionTurn", undo, unit.luaData)
    undoSave("lastActionId", undo, unit.luaData)
    SetUnitAction(unit)
    
    UnitNoEntrenchTurn(unit)

    AdviceMove(unit)

    TriggerMoveEvents(unit)
  end
  
end

function undoLoad(name, undo, table)
  table[name] = undo:Get(name)
end

function OnUnitMoveUndo(unit, undo)
  LogUi("Move UNDO unit#" .. tostring(unit.id))
  
  if undo:GetBool("railMove") then
    undoLoad("railmovesCurrent", undo, unit.faction.luaData)
    undoLoad("repaired", undo, unit.luaData)
    undoLoad("upgraded", undo, unit.luaData)

  end
  
  unit.ap = undo:GetInt("ap")
  
  if undo:Has("gameplay.advanceUnit") then
    gameplay.advanceUnit = GetUnitById(undo:GetInt("gameplay.advanceUnit"))
    gameplay.advanceHex = GetHex(undo:GetInt("gameplay.advanceHex"))
  end
  
  if unit.alive then
    unit.efficicncy = undo:GetInt("efficiency")

    undoLoad("lastAction", undo, unit.luaData)
    undoLoad("lastActionTurn", undo, unit.luaData)
    undoLoad("lastActionId", undo, unit.luaData)
  end
  
  GetPlayer():DeselectUnit()
 
end

function MoveCommander(unit, from)

  if unit.commander ~= nil then
    local range = data.commanders[unit.commander.id+1].range

    -- Clear previously affected units
    if from ~= nil then
      for x = -range, range do
        for y = -range, range do
          local hex = GetHex(from.x + x, from.y + y)
          if hex ~= nil and hex.unit ~= nil and hex.unit.alive then
            if unit.commanderInfluence == unit.commander.id then
              --unit.commanderInfluence = nil
            end
          end
        end
      end
    end

    -- Affect units now in range
    for x = -range, range do
      for y = -range, range do
        local hex = GetHex(unit.hex.x + x, unit.hex.y + y)
        if hex ~= nil and hex.unit ~= nil and hex.unit.alive then
          if hex.unit.faction.id == unit.faction.id then
            if unit.hex:Distance(hex) <= range then
              unit.commanderInfluence = unit.commander.id
            end
          end
        end
      end
    end

  end

end


-- Play movement sound
function MoveSound(unit, moveType)
  local sfx = {}
  local counter = 0

  if moveType ~= 1 then
    local i
    for i=0,3 do
      if engine:FileExists("sound", unit.prototype.name .. "_move_" .. (i+1) .. ".wav") then
        sfx[i] = LoadSample(unit.prototype.name .. "_move_" .. (i+1) .. ".wav")
        counter = counter + 1
      else
        break
      end
    end
  end
  if moveType == 1 then
    local i
    for i=0,2 do
      sfx[i] = LoadSample("train_move_" .. (i+1) .. ".wav")
      counter = counter + 1
    end
  end

  if counter > 0 then
    PlaySample(sfx[math.random(0, (counter - 1))])
  end
end

-- Check if convoy reaches port
function MoveConvoy(unit)

  if unit.alive and unit.hex.construction ~= nil and unit.prototype.name == "convoy" then
    if unit.hex.construction.type == Construction.TYPE_PORT then
      unit.faction:ConsumeProductionPoints(-unit.hp)
      unit:Kill()
      unit:Damage(unit.hp)
    end
  end

end

function CanUnitAttackAfterMove(unit)
  if unit.prototype.lua.class == "air" or unit.prototype.lua.class == "artillery" or unit.prototype.name == "transport" then
    return false
  else
    return true
  end
end

-- Have to pass actual embark function because it differs between AI and player(UI)
function EmbarkUnit(unit, hex, embarkFunc)
  embarkFunc(unit, hex)
  local result = unit.state == Unit.STATE_EMBARKED

  if result ~= false then
    local cargoHp = unit.hp
    if cargoHp ~= hex.unit.hp then
      hex.unit:Damage(hex.unit.hp - cargoHp)
    end

    UseTransportmove(unit.faction)

    -- Sound effect
    if playerAlliance:IsVisible(hex) then
      PlaySample(sEmbark)
    end
  end

  return result
end

-- Have to pass actual disembark function because it differs between AI and player(UI)
function DisembarkUnit(unit, hex, disembarkFunc)
  disembarkFunc(unit, hex)
  local result = unit.state ~= Unit.STATE_EMBARKED

  if result then
    hex.unit.ap = 0

    -- Sound effect
    if playerAlliance:IsVisible(hex) then
      PlaySample(sDisembark)
    end
  end

  return result
end


function IsTerrainValidForUnit(hex, unit)
  for _, chassis in pairs(data.chassis) do
    if chassis.name == unit.prototype.lua.chassis then
      if unit.prototype.lua.levels[1].mp >= chassis.costs[hex.terrain.name] then
        if unit.prototype.lua.chassis ~= "rail" or hex.hasRailroad then
          return true
        end
      end
      return false
    end
  end
end

function GetMoveEfficiencyLoss(unit, hex, railMove)

  if hex == nil then
    hex = unit.hex
  end

  local loss = 0

  if railMove then
    -- Efficiency loss
    loss = gameplay.RailmoveEfficiencyLoss
  else

    -- Land and Air units lose efficiency from movement
    if unit.type ~= Unit.NAVAL then
     loss = gameplay.MoveEfficiencyLoss

      -- Winter effect
      if hex ~= nil and GetClimate() > hex.climate then
        loss = loss + gameplay.WinterEfficiencyPenalty
      end
    end
  end

  return loss
end
